# Load dataset
data <- readRDS("data_basic.rds")

# Functions

# (min-max normalization)
normalize.MinMax <- function(x, range = c(0,1), reverse = FALSE) {
    x <- (x - min(x)) /(max(x)-min(x))
    x <- (range[2]-range[1])*x + range[1]
    if(reverse) x <- range[2] - x
    return(x)
}

# (geomorphology)
calc.geomorphology <- function(data){
    dat <- data.frame(
        geomC1.pct = data$geomC1/data$area.ha,
        geomC2.pct = data$geomC2/data$area.ha,
        geomC3.pct = data$geomC3/data$area.ha
    )
    dat$geomCT.pct <- normalize.MinMax(apply(dat,1,sum)) #!? normalize?
    dat$geomorphology <- normalize.MinMax(dat$geomCT.pct/100, range=c(0,3))
    dat$mb <- data$mb
    return(dat)
}

# (demography)
calc.demography <- function(data){
    data$over300pop.pct <- with(data, over300pop/population)
    data$workagepop.pct <- with(data, workagepop/population)
    data$over65pop.pct <- with(data, over65pop/population)
    data$depop.cens <- data$depop
    data$depop.cens[data$depop.cens >= 100] <- 100
    dat <- data.frame(
        over300pop.pt = normalize.MinMax(data$over300pop.pct,range=c(0,2)),
        workagepop.pt = normalize.MinMax(data$workagepop.pct, reverse=TRUE),
        over65pop.pt = normalize.MinMax(data$over65pop.pct),
        depop.pt = normalize.MinMax(data$depop.cens, reverse=TRUE)
    )
    dat$demography.pt <- apply(dat,1,sum)
    dat$demography <- normalize.MinMax(dat$demography.pt)
    dat$mb <- data$mb
    return(dat)
}

# (climate)
calc.climate <- function(data){
    dat <- data.frame(
        precipitation.pt = normalize.MinMax(data$precipitation),
        airtemp.pt = normalize.MinMax(data$airtemp,reverse=TRUE),
        snow.pt = normalize.MinMax(data$snow, range=c(0,2))
    )
    dat$climate.pt <- apply(dat,1,sum)
    dat$climate <- normalize.MinMax(dat$climate.pt)
    dat$mb <- data$mb
    return(dat)
}

# (infrastructure)
calc.infrastructure <- function(data, rec.school = "0=1;1=.5;2=.4;3=.3;4=.2;5:'high'=0", rec.famed = rec.school){
    dat <- data.frame(
        school.pt = car::Recode(data$school,rec.school),
        famed.pt = car::Recode(data$famed,rec.famed),
        roadden.pt = normalize.MinMax(with(data, roadlen / area.km2), reverse = TRUE)
    )
    dat$infrastructure.pt <- apply(dat,1,sum)
    dat$infrastructure <- normalize.MinMax(dat$infrastructure.pt)
    dat$mb <- data$mb
    return(dat)
}



# Replication

# (calculate subindices, merge with 'data')
lst <- list(
    calc.geomorphology(data),
    calc.demography(data),
    calc.climate(data),
    calc.infrastructure(data)
)
df <- Reduce(function(...) merge(...,by="mb"), lst)
data <- merge(data,df,by="mb")

# (calculate bpp index)
data$bpp <- with(data, geomorphology+climate+demography+infrastructure)

# (calculate bpp indicators, replicated and original)
data$bpp.ind <- factor(with(data, bpp > 2.5 & !(island == "yes" | (coastal == "yes" & popden > 20))),levels=c(FALSE,TRUE), labels=c("no","yes"))
data$orig.bpp.ind <- factor(with(data, orig.bpp > 2.5 & !(island == "yes" | (coastal == "yes" & popden > 20))),levels=c(FALSE,TRUE), labels=c("no","yes"))

# (table original and replicated indicators)
addmargins(xtabs(~bpp.ind+orig.bpp.ind, data))

# (plot replicated and original bpp index)
plot(data[c("bpp","orig.bpp")])
abline(h=2.5,v=2.5,lty=3)

# (check ranges of numeric variables)
apply(data[sapply(data,is.numeric)],2,range)

# Output
#data <- data[order(data$bpp, decreasing=TRUE),]
#data$mb <- paste0("'",data$mb)
#write.csv(data,"bpp_replication.csv",row.names=FALSE)

